@@ -28,6 +28,7 @@
 #include <string.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
 #include <telepathy-glib/util.h>
@@ -105,6 +106,10 @@ static void             contact_list_store_members_changed_cb        (EmpathyCon
 								      gchar                         *message,
 								      gboolean                       is_member,
 								      EmpathyContactListStore       *store);
+static void             contact_list_store_favourites_changed_cb     (EmpathyContactList            *list_iface,
+								      EmpathyContact                *contact,
+								      gboolean                       is_favourite,
+								      EmpathyContactListStore       *store);
 static void             contact_list_store_member_renamed_cb         (EmpathyContactList            *list_iface,
 								      EmpathyContact                *old_contact,
 								      EmpathyContact                *new_contact,
@@ -142,7 +147,8 @@ static void             contact_list_store_get_group                 (EmpathyCon
 								      const gchar                   *name,
 								      GtkTreeIter                   *iter_group_to_set,
 								      GtkTreeIter                   *iter_separator_to_set,
-								      gboolean                      *created);
+								      gboolean                      *created,
+								      gboolean                      is_fake_group);
 static gint             contact_list_store_state_sort_func           (GtkTreeModel                  *model,
 								      GtkTreeIter                   *iter_a,
 								      GtkTreeIter                   *iter_b,
@@ -191,6 +197,10 @@ contact_list_store_iface_setup (gpointer user_data)
 			  "members-changed",
 			  G_CALLBACK (contact_list_store_members_changed_cb),
 			  store);
+	g_signal_connect (priv->list,
+			  "favourites-changed",
+			  G_CALLBACK (contact_list_store_favourites_changed_cb),
+			  store);
 	g_signal_connect (priv->list,
 			  "groups-changed",
 			  G_CALLBACK (contact_list_store_groups_changed_cb),
@@ -337,6 +347,9 @@ contact_list_store_dispose (GObject *object)
 	g_signal_handlers_disconnect_by_func (priv->list,
 					      G_CALLBACK (contact_list_store_members_changed_cb),
 					      object);
+	g_signal_handlers_disconnect_by_func (priv->list,
+					      G_CALLBACK (contact_list_store_favourites_changed_cb),
+					      object);
 	g_signal_handlers_disconnect_by_func (priv->list,
 					      G_CALLBACK (contact_list_store_groups_changed_cb),
 					      object);
@@ -715,11 +728,13 @@ empathy_contact_list_store_row_separator_func (GtkTreeModel *model,
 gchar *
 empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 					    GtkTreePath  *path,
-					    gboolean     *path_is_group)
+					    gboolean     *path_is_group,
+					    gboolean     *is_fake_group)
 {
 	GtkTreeIter  parent_iter, iter;
 	gchar       *name = NULL;
 	gboolean     is_group;
+	gboolean     fake;
 
 	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), NULL);
 
@@ -749,6 +764,7 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 		gtk_tree_model_get (model, &iter,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, &is_group,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake,
 				    -1);
 		if (!is_group) {
 			g_free (name);
@@ -760,6 +776,9 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 		*path_is_group = TRUE;
 	}
 
+	if (is_fake_group != NULL)
+		*is_fake_group = fake;
+
 	return name;
 }
 
@@ -824,6 +843,7 @@ contact_list_store_setup (EmpathyContactListStore *store)
 		G_TYPE_BOOLEAN,       /* Can make audio calls */
 		G_TYPE_BOOLEAN,       /* Can make video calls */
 		EMPATHY_TYPE_CONTACT_LIST_FLAGS, /* Flags */
+		G_TYPE_BOOLEAN,       /* Is a fake group */
 	};
 
 	priv = GET_PRIV (store);
@@ -916,6 +936,25 @@ contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 	}
 }
 
+static void
+contact_list_store_favourites_changed_cb (EmpathyContactList      *list_iface,
+					  EmpathyContact          *contact,
+					  gboolean                 is_favourite,
+					  EmpathyContactListStore *store)
+{
+	EmpathyContactListStorePriv *priv;
+
+	priv = GET_PRIV (store);
+
+	DEBUG ("Contact %s (%d) is %s a favourite",
+		empathy_contact_get_id (contact),
+		empathy_contact_get_handle (contact),
+		is_favourite ? "now" : "no longer");
+
+	contact_list_store_remove_contact (store, contact);
+	contact_list_store_add_contact (store, contact);
+}
+
 static void
 contact_list_store_member_renamed_cb (EmpathyContactList      *list_iface,
 				      EmpathyContact          *old_contact,
@@ -968,6 +1007,27 @@ contact_list_store_groups_changed_cb (EmpathyContactList      *list_iface,
 	priv->show_active = show_active;
 }
 
+static void
+add_contact_to_store (GtkTreeStore *store,
+		      GtkTreeIter *iter,
+		      EmpathyContact *contact,
+		      EmpathyContactListFlags flags)
+{
+	gtk_tree_store_set (store, iter,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
+			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
+			      empathy_contact_get_capabilities (contact) &
+			        EMPATHY_CAPABILITIES_AUDIO,
+			    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
+			      empathy_contact_get_capabilities (contact) &
+			        EMPATHY_CAPABILITIES_VIDEO,
+			    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
+			    -1);
+}
+
 static void
 contact_list_store_add_contact (EmpathyContactListStore *store,
 				EmpathyContact          *contact)
@@ -994,8 +1054,18 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		flags = empathy_contact_manager_get_flags_for_connection (
 			EMPATHY_CONTACT_MANAGER (priv->list), connection);
 	}
-	/* If no groups just add it at the top level. */
 	if (!groups) {
+#if HAVE_FAVOURITE_CONTACTS
+		GtkTreeIter iter_group;
+
+		contact_list_store_get_group (store, EMPATHY_CONTACT_LIST_STORE_UNGROUPED,
+			&iter_group, NULL, NULL, TRUE);
+
+		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
+					     &iter_group, NULL);
+#else
+		/* FIXME: remove this in 2.31.x */
+		/* If no groups just add it at the top level. */
 		GtkTreeModel *model = GTK_TREE_MODEL (store);
 
 		if (gtk_tree_model_get_iter_first (model, &iter)) do {
@@ -1014,48 +1084,41 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		} while (gtk_tree_model_iter_next (model, &iter));
 
 		gtk_tree_store_append (GTK_TREE_STORE (store), &iter, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
-				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_AUDIO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_VIDEO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
-				    -1);
+#endif
+
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
 	}
 
 	/* Else add to each group. */
 	for (l = groups; l; l = l->next) {
 		GtkTreeIter iter_group;
 
-		contact_list_store_get_group (store, l->data, &iter_group, NULL, NULL);
+		contact_list_store_get_group (store, l->data, &iter_group, NULL, NULL, FALSE);
 
 		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
 					     &iter_group, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter,
-				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_AUDIO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
-				      empathy_contact_get_capabilities (contact) &
-				        EMPATHY_CAPABILITIES_VIDEO,
-				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
-				    -1);
+
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
 		g_free (l->data);
 	}
 	g_list_free (groups);
 
-	contact_list_store_contact_update (store, contact);
+#ifdef HAVE_FAVOURITE_CONTACTS
+	if (empathy_contact_list_is_favourite (priv->list, contact)) {
+	/* Add contact to the fake 'Favorites' group */
+		GtkTreeIter iter_group;
 
+		contact_list_store_get_group (store, EMPATHY_CONTACT_LIST_STORE_FAVORITE,
+			&iter_group, NULL, NULL, TRUE);
+
+		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
+					     &iter_group, NULL);
+
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+	}
+#endif
+
+	contact_list_store_contact_update (store, contact);
 }
 
 static void
@@ -1112,7 +1175,6 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	gboolean                    do_set_active = FALSE;
 	gboolean                    do_set_refresh = FALSE;
 	gboolean                    show_avatar = FALSE;
-	gboolean                    show_protocol = FALSE;
 	GdkPixbuf                  *pixbuf_avatar;
 	GdkPixbuf                  *pixbuf_status;
 
@@ -1207,9 +1269,6 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	if (priv->show_avatars && !priv->is_compact) {
 		show_avatar = TRUE;
 	}
-	if (priv->show_protocols && !priv->is_compact) {
-		show_protocol = TRUE;
-	}
 	pixbuf_avatar = empathy_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
 	pixbuf_status = contact_list_store_get_contact_status_icon (store, contact);
 	for (l = iters; l && set_model; l = l->next) {
@@ -1393,7 +1452,8 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			      const gchar            *name,
 			      GtkTreeIter            *iter_group_to_set,
 			      GtkTreeIter            *iter_separator_to_set,
-			      gboolean               *created)
+			      gboolean               *created,
+			      gboolean               is_fake_group)
 {
 	EmpathyContactListStorePriv *priv;
 	GtkTreeModel                *model;
@@ -1424,6 +1484,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, TRUE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ACTIVE, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
+				    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, is_fake_group,
 				    -1);
 
 		if (iter_group_to_set) {
@@ -1465,45 +1526,78 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 	}
 }
 
+static gint
+compare_separator_and_groups (gboolean is_separator_a,
+			      gboolean is_separator_b,
+			      const gchar *name_a,
+			      const gchar *name_b,
+			      EmpathyContact *contact_a,
+			      EmpathyContact *contact_b,
+			      gboolean fake_group_a,
+			      gboolean fake_group_b)
+{
+	if (is_separator_a || is_separator_b) {
+		/* We have at least one separator */
+		if (is_separator_a) {
+			return -1;
+		} else if (is_separator_b) {
+			return 1;
+		}
+	}
+
+	/* One group and one contact */
+	if (!contact_a && contact_b) {
+		return 1;
+	} else if (contact_a && !contact_b) {
+		return -1;
+	} else if (!contact_a && !contact_b) {
+		/* Two groups. The 'Ungrouped' fake group is display at the bottom of the
+		 * contact list and the 'Favorites' at the top. */
+		if (fake_group_a && !tp_strdiff (name_a, EMPATHY_CONTACT_LIST_STORE_UNGROUPED))
+			return 1;
+		else if (fake_group_b && !tp_strdiff (name_b, EMPATHY_CONTACT_LIST_STORE_UNGROUPED))
+			return -1;
+		else if (fake_group_a && !tp_strdiff (name_a, EMPATHY_CONTACT_LIST_STORE_FAVORITE))
+			return -1;
+		else if (fake_group_b && !tp_strdiff (name_b, EMPATHY_CONTACT_LIST_STORE_FAVORITE))
+			return 1;
+		else
+			return g_utf8_collate (name_a, name_b);
+	}
+
+	/* Two contacts, ordering depends of the sorting policy */
+	return 0;
+}
+
 static gint
 contact_list_store_state_sort_func (GtkTreeModel *model,
 				    GtkTreeIter  *iter_a,
 				    GtkTreeIter  *iter_b,
 				    gpointer      user_data)
 {
-	gint            ret_val = 0;
+	gint            ret_val;
 	gchar          *name_a, *name_b;
 	gboolean        is_separator_a, is_separator_b;
 	EmpathyContact *contact_a, *contact_b;
+	gboolean       fake_group_a, fake_group_b;
 
 	gtk_tree_model_get (model, iter_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_a,
 			    -1);
 	gtk_tree_model_get (model, iter_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	/* Separator or group? */
-	if (is_separator_a || is_separator_b) {
-		if (is_separator_a) {
-			ret_val = -1;
-		} else if (is_separator_b) {
-			ret_val = 1;
-		}
-	} else if (!contact_a && contact_b) {
-		ret_val = 1;
-	} else if (contact_a && !contact_b) {
-		ret_val = -1;
-	} else if (!contact_a && !contact_b) {
-		/* Handle groups */
-		ret_val = g_utf8_collate (name_a, name_b);
-	}
+	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
 
-	if (ret_val) {
+	if (ret_val != 0) {
 		goto free_and_out;
 	}
 
@@ -1542,38 +1636,28 @@ contact_list_store_name_sort_func (GtkTreeModel *model,
 {
 	gchar         *name_a, *name_b;
 	EmpathyContact *contact_a, *contact_b;
-	gboolean       is_separator_a, is_separator_b;
+	gboolean       is_separator_a = FALSE, is_separator_b = FALSE;
 	gint           ret_val;
+	gboolean       fake_group_a, fake_group_b;
 
 	gtk_tree_model_get (model, iter_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_a,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_a,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_a,
 			    -1);
 	gtk_tree_model_get (model, iter_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact_b,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, &is_separator_b,
+			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	/* If contact is NULL it means it's a group. */
+	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
 
-	if (is_separator_a || is_separator_b) {
-		if (is_separator_a) {
-			ret_val = -1;
-		} else if (is_separator_b) {
-			ret_val = 1;
-		}
-	} else if (!contact_a && contact_b) {
-		ret_val = 1;
-	} else if (contact_a && !contact_b) {
-		ret_val = -1;
-	} else {
+	if (ret_val == 0)
 		ret_val = g_utf8_collate (name_a, name_b);
-	}
-
-	g_free (name_a);
-	g_free (name_b);
 
 	if (contact_a) {
 		g_object_unref (contact_a);
@@ -1645,7 +1729,6 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 {
 	EmpathyContactListStorePriv *priv;
 	gboolean                     show_avatar = FALSE;
-	gboolean                     show_protocol = FALSE;
 	EmpathyContact              *contact;
 	GdkPixbuf                   *pixbuf_status;
 
@@ -1654,9 +1737,6 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 	if (priv->show_avatars && !priv->is_compact) {
 		show_avatar = TRUE;
 	}
-	if (priv->show_protocols && !priv->is_compact) {
-		show_protocol = TRUE;
-	}
 
 	gtk_tree_model_get (model, iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
@@ -1687,14 +1767,10 @@ contact_list_store_get_contact_status_icon_with_icon_name (
 	EmpathyContactListStorePriv *priv;
 	const gchar                 *protocol_name = NULL;
 	gchar                       *icon_name = NULL;
-	gboolean                     show_protocol = FALSE;
 
 	priv = GET_PRIV (store);
 
-	if (priv->show_protocols && !priv->is_compact) {
-		show_protocol = TRUE;
-	}
-	if (show_protocol) {
+	if (priv->show_protocols) {
 		protocol_name = empathy_protocol_name_for_contact (contact);
 		icon_name = g_strdup_printf ("%s-%s", status_icon_name, protocol_name);
 	} else {
@@ -1704,16 +1780,14 @@ contact_list_store_get_contact_status_icon_with_icon_name (
 	if (pixbuf_status == NULL) {
 		pixbuf_status = empathy_pixbuf_contact_status_icon_with_icon_name (contact,
 				    status_icon_name,
-				    show_protocol);
+				    priv->show_protocols);
 		if (pixbuf_status != NULL) {
 			g_hash_table_insert (priv->status_icons,
 			    g_strdup (icon_name),
 			    pixbuf_status);
-			DEBUG( "Created status icon %s\n", icon_name);
 		}
-	} else {
-		DEBUG( "retrieved from cache status icon %s\n", icon_name);
 	}
+
 	g_free (icon_name);
 	return pixbuf_status;
 }
@@ -1726,6 +1800,9 @@ contact_list_store_get_contact_status_icon (EmpathyContactListStore *store,
 	const gchar                 *status_icon_name = NULL;
 
 	status_icon_name = empathy_icon_name_for_contact (contact);
+	if (status_icon_name == NULL)
+		return NULL;
+
 	pixbuf_status = contact_list_store_get_contact_status_icon_with_icon_name (
 			    store,
 			    contact,
