@@ -63,9 +63,13 @@ typedef enum {
 	EMPATHY_CONTACT_LIST_STORE_COL_CAN_AUDIO_CALL,
 	EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
 	EMPATHY_CONTACT_LIST_STORE_COL_FLAGS,
-	EMPATHY_CONTACT_LIST_STORE_COL_COUNT
+	EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP,
+	EMPATHY_CONTACT_LIST_STORE_COL_COUNT,
 } EmpathyContactListStoreCol;
 
+#define EMPATHY_CONTACT_LIST_STORE_UNGROUPED _("Ungrouped")
+#define EMPATHY_CONTACT_LIST_STORE_FAVORITE  _("Favorite People")
+
 struct _EmpathyContactListStore {
 	GtkTreeStore parent;
 	gpointer priv;
@@ -101,7 +105,8 @@ gboolean                   empathy_contact_list_store_row_separator_func (GtkTre
 									 gpointer                    data);
 gchar *                    empathy_contact_list_store_get_parent_group   (GtkTreeModel               *model,
 									 GtkTreePath                *path,
-									 gboolean                   *path_is_group);
+									 gboolean                   *path_is_group,
+									 gboolean                   *is_fake_group);
 gboolean                   empathy_contact_list_store_search_equal_func  (GtkTreeModel               *model,
 									 gint                        column,
 									 const gchar                *key,
