@@ -40,6 +40,8 @@
 #include "empathy-conf.h"
 #include "empathy-ui-utils.h"
 #include "empathy-plist.h"
+#include "empathy-string-parser.h"
+#include "empathy-images.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -47,8 +49,8 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyThemeAdium)
 
 /* GConf key containing current value of font */
-#define EMPATHY_GCONF_FONT_KEY_NAME       "/desktop/gnome/interface/font_name"
-#define BORING_DPI_DEFAULT                96
+#define EMPATHY_GCONF_FONT_KEY_NAME "/desktop/gnome/interface/document_font_name"
+#define BORING_DPI_DEFAULT 96
 
 /* "Join" consecutive messages with timestamps within five minutes */
 #define MESSAGE_JOIN_PERIOD 5*60
@@ -220,30 +222,6 @@ theme_adium_match_newline (const gchar *text,
 				      sub_parsers, user_data);
 }
 
-static void
-theme_adium_replace_link (const gchar *text,
-			  gssize len,
-			  gpointer match_data,
-			  gpointer user_data)
-{
-	GString *string = user_data;
-	gchar *real_url;
-	gchar *escaped;
-
-	real_url = empathy_make_absolute_url_len (text, len);
-
-	/* The thing we are making a link of may contain
-	 * characters which need escaping */
-	escaped = g_markup_escape_text (text, len);
-
-	/* Append the link inside <a href=""></a> tag */
-	g_string_append_printf (string, "<a href=\"%s\">%s</a>",
-				real_url, escaped);
-
-	g_free (real_url);
-	g_free (escaped);
-}
-
 static void
 theme_adium_replace_smiley (const gchar *text,
 			    gssize len,
@@ -259,32 +237,18 @@ theme_adium_replace_smiley (const gchar *text,
 				hit->path, (int)len, text, (int)len, text);
 }
 
-static void
-theme_adium_replace_escaped (const gchar *text,
-			     gssize len,
-			     gpointer match_data,
-			     gpointer user_data)
-{
-	GString *string = user_data;
-	gchar *escaped;
-
-	escaped = g_markup_escape_text (text, len);
-	g_string_append (string, escaped);
-	g_free (escaped);
-}
-
 static EmpathyStringParser string_parsers[] = {
-	{empathy_string_match_link, theme_adium_replace_link},
+	{empathy_string_match_link, empathy_string_replace_link},
 	{theme_adium_match_newline, NULL},
-	{empathy_string_match_all, theme_adium_replace_escaped},
+	{empathy_string_match_all, empathy_string_replace_escaped},
 	{NULL, NULL}
 };
 
 static EmpathyStringParser string_parsers_with_smiley[] = {
-	{empathy_string_match_link, theme_adium_replace_link},
+	{empathy_string_match_link, empathy_string_replace_link},
 	{empathy_string_match_smiley, theme_adium_replace_smiley},
 	{theme_adium_match_newline, NULL},
-	{empathy_string_match_all, theme_adium_replace_escaped},
+	{empathy_string_match_all, empathy_string_replace_escaped},
 	{NULL, NULL}
 };
 
@@ -528,7 +492,7 @@ theme_adium_append_message (EmpathyChatView *view,
 		if (!avatar_filename) {
 			if (!priv->data->default_avatar_filename) {
 				priv->data->default_avatar_filename =
-					empathy_filename_from_icon_name ("stock_person",
+					empathy_filename_from_icon_name (EMPATHY_IMAGE_AVATAR_DEFAULT,
 									 GTK_ICON_SIZE_DIALOG);
 			}
 			avatar_filename = priv->data->default_avatar_filename;
