@@ -27,6 +27,9 @@
 
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
+#ifdef ENABEL_TPL
+#include <telepathy-logger/contact.h>
+#endif /* ENABLE_TPL */
 
 #include "empathy-contact.h"
 #include "empathy-utils.h"
@@ -408,6 +411,33 @@ empathy_contact_new (TpContact *tp_contact)
       NULL);
 }
 
+#ifdef ENABLE_TPL
+EmpathyContact *
+empathy_contact_from_tpl_contact (TpAccount *account,
+    TplContact *tpl_contact)
+{
+  EmpathyContact *retval;
+  gboolean is_user;
+
+  g_return_val_if_fail (TPL_IS_CONTACT (tpl_contact), NULL);
+
+  is_user = (TPL_CONTACT_USER == tpl_contact_get_contact_type (tpl_contact));
+
+  retval = g_object_new (EMPATHY_TYPE_CONTACT,
+      "id", tpl_contact_get_alias (tpl_contact),
+      "name", tpl_contact_get_identifier (tpl_contact),
+      "account", account,
+      "is-user", is_user,
+      NULL);
+
+  if (!EMP_STR_EMPTY (tpl_contact_get_avatar_token (tpl_contact)))
+    empathy_contact_load_avatar_cache (retval,
+        tpl_contact_get_avatar_token (tpl_contact));
+
+  return retval;
+}
+#endif /* ENABLE_TPL */
+
 EmpathyContact *
 empathy_contact_new_for_log (TpAccount *account,
                              const gchar *id,
@@ -483,18 +513,21 @@ const gchar *
 empathy_contact_get_name (EmpathyContact *contact)
 {
   EmpathyContactPriv *priv;
+  const gchar        *alias;
 
   g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
   priv = GET_PRIV (contact);
 
   if (priv->tp_contact != NULL)
-    return tp_contact_get_alias (priv->tp_contact);
+    alias = tp_contact_get_alias (priv->tp_contact);
+  else
+    alias = priv->name;
 
-  if (EMP_STR_EMPTY (priv->name))
-      return empathy_contact_get_id (contact);
-
-  return priv->name;
+  if (!EMP_STR_EMPTY (alias))
+    return alias;
+  else
+    return empathy_contact_get_id (contact);
 }
 
 void
@@ -1016,6 +1049,7 @@ empathy_avatar_unref (EmpathyAvatar *avatar)
       g_free (avatar->data);
       g_free (avatar->format);
       g_free (avatar->token);
+      g_free (avatar->filename);
       g_slice_free (EmpathyAvatar, avatar);
     }
 }
