@@ -335,8 +335,31 @@ dispatcher_connection_invalidated_cb (TpConnection *connection,
                                       EmpathyDispatcher *self)
 {
   EmpathyDispatcherPriv *priv = GET_PRIV (self);
+  ConnectionData *connection_data;
 
   DEBUG ("Error: %s", message);
+
+  /* Terminate pending requests, if any */
+  connection_data = g_hash_table_lookup (priv->connections, connection);
+  if (connection_data != NULL)
+    {
+      GList *l;
+      GError *error = NULL;
+
+      error = g_error_new_literal (domain, code, message);
+
+      for (l = connection_data->outstanding_requests; l != NULL;
+          l = g_list_next (l))
+        {
+          DispatcherRequestData *request_data = l->data;
+
+          if (request_data->cb != NULL)
+            request_data->cb (NULL, error, request_data->user_data);
+        }
+
+      g_error_free (error);
+    }
+
   g_hash_table_remove (priv->connections, connection);
 }
 
@@ -389,6 +412,8 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
 
   connection = tp_channel_borrow_connection (TP_CHANNEL (proxy));
 
+  priv->channels = g_list_remove (priv->channels, proxy);
+
   cd = g_hash_table_lookup (priv->connections, connection);
   /* Connection itself invalidated? */
   if (cd == NULL)
@@ -400,8 +425,6 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
 
   g_hash_table_remove (cd->dispatched_channels, object_path);
   g_hash_table_remove (cd->dispatching_channels, object_path);
-
-  priv->channels = g_list_remove (priv->channels, proxy);
 }
 
 static void
@@ -1423,6 +1446,17 @@ empathy_dispatcher_chat_with_contact (EmpathyContact *contact,
 
   connection = empathy_contact_get_connection (contact);
   connection_data = g_hash_table_lookup (priv->connections, connection);
+  if (connection_data == NULL)
+    {
+      /* Connection has been invalidated */
+      if (callback != NULL)
+        {
+          GError error = { TP_DBUS_ERRORS, TP_DBUS_ERROR_PROXY_UNREFERENCED,
+              "Connection has been invalidated" };
+          callback (NULL, &error, user_data);
+        }
+      goto out;
+    }
 
   /* The contact handle might not be known yet */
   request_data = new_dispatcher_request_data (self, connection,
@@ -1435,6 +1469,7 @@ empathy_dispatcher_chat_with_contact (EmpathyContact *contact,
 
   dispatcher_request_channel (request_data);
 
+out:
   g_object_unref (self);
 }
 
@@ -1456,8 +1491,12 @@ dispatcher_chat_with_contact_id_cb (EmpathyTpContactFactory *factory,
 
   if (error)
     {
-      /* FIXME: Should call data->callback with the error */
       DEBUG ("Error: %s", error->message);
+
+      if (data->callback != NULL)
+        {
+          data->callback (NULL, error, data->user_data);
+        }
     }
   else
     {
