@@ -61,7 +61,6 @@ typedef struct {
 
 GType          empathy_tp_chat_get_type             (void) G_GNUC_CONST;
 EmpathyTpChat *empathy_tp_chat_new                  (TpChannel          *channel);
-void           empathy_tp_chat_close                (EmpathyTpChat      *chat);
 const gchar *  empathy_tp_chat_get_id               (EmpathyTpChat      *chat);
 EmpathyContact *empathy_tp_chat_get_remote_contact   (EmpathyTpChat      *chat);
 TpChannel *    empathy_tp_chat_get_channel          (EmpathyTpChat      *chat);
@@ -85,6 +84,7 @@ void           empathy_tp_chat_acknowledge_message (EmpathyTpChat *chat,
 						     EmpathyMessage *message);
 void           empathy_tp_chat_acknowledge_messages (EmpathyTpChat *chat,
 						     const GSList *messages);
+void           empathy_tp_chat_acknowledge_all_messages (EmpathyTpChat *chat);
 
 gboolean       empathy_tp_chat_password_needed      (EmpathyTpChat *chat);
 
@@ -98,6 +98,8 @@ gboolean       empathy_tp_chat_provide_password_finish (EmpathyTpChat *chat,
 							GError **error);
 gboolean       empathy_tp_chat_can_add_contact (EmpathyTpChat *self);
 
+void           empathy_tp_chat_leave                (EmpathyTpChat      *chat);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_TP_CHAT_H__ */
