@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright © 2000 Eazel, Inc.
  * Copyright © 2002-2004 Marco Pesenti Gritti
  * Copyright © 2004, 2006 Christian Persch
@@ -20,7 +20,7 @@
  * Author: Andy Hertzfeld <andy@eazel.com>
  *
  * Ephy port by Marco Pesenti Gritti <marco@it.gnome.org>
- * 
+ *
  * $Id: ephy-spinner.c 2114 2006-12-25 12:15:00Z mr $
  */
 
@@ -34,9 +34,7 @@
 #define STOP_PROFILER(name)
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtksettings.h>
+#include <gtk/gtk.h>
 
 /* Spinner cache implementation */
 
@@ -394,7 +392,7 @@ ephy_spinner_cache_get_images (EphySpinnerCache *cache,
 
 	LOG ("Getting animation images for screen %p at size %d", screen, icon_size);
 
-	g_return_val_if_fail (icon_size >= 0 && icon_size < LAST_ICON_SIZE, NULL);
+	g_return_val_if_fail (icon_size < LAST_ICON_SIZE, NULL);
 
 	/* Backward compat: "invalid" meant "native" size which doesn't exist anymore */
 	if (icon_size == GTK_ICON_SIZE_INVALID)
@@ -455,7 +453,7 @@ ephy_spinner_cache_init (EphySpinnerCache *cache)
 static void
 ephy_spinner_cache_finalize (GObject *object)
 {
-	EphySpinnerCache *cache = EPHY_SPINNER_CACHE (object); 
+	EphySpinnerCache *cache = EPHY_SPINNER_CACHE (object);
 	EphySpinnerCachePrivate *priv = cache->priv;
 
 	g_hash_table_destroy (priv->hash);
@@ -603,7 +601,7 @@ ephy_spinner_init (EphySpinner *spinner)
 
 	details = spinner->details = EPHY_SPINNER_GET_PRIVATE (spinner);
 
-	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (spinner), GTK_NO_WINDOW);
+	gtk_widget_set_has_window (GTK_WIDGET (spinner), FALSE);
 
 	details->cache = ephy_spinner_cache_ref ();
 	details->size = GTK_ICON_SIZE_DIALOG;
@@ -623,8 +621,9 @@ ephy_spinner_expose (GtkWidget *widget,
 	GdkGC *gc;
 	int x_offset, y_offset, width, height;
 	GdkRectangle pix_area, dest;
+	GtkAllocation allocation;
 
-	if (!GTK_WIDGET_DRAWABLE (spinner))
+	if (!gtk_widget_is_drawable (GTK_WIDGET (spinner)))
 	{
 		return FALSE;
 	}
@@ -644,8 +643,7 @@ ephy_spinner_expose (GtkWidget *widget,
 	/* Otherwise |images| will be NULL anyway */
 	g_assert (images->n_animation_pixbufs > 0);
 		
-	g_assert (details->current_image >= 0 &&
-		  details->current_image < images->n_animation_pixbufs);
+	g_assert (details->current_image < images->n_animation_pixbufs);
 
 	pixbuf = images->animation_pixbufs[details->current_image];
 
@@ -655,11 +653,12 @@ ephy_spinner_expose (GtkWidget *widget,
 	height = gdk_pixbuf_get_height (pixbuf);
 
 	/* Compute the offsets for the image centered on our allocation */
-	x_offset = (widget->allocation.width - width) / 2;
-	y_offset = (widget->allocation.height - height) / 2;
+	gtk_widget_get_allocation (widget, &allocation);
+	x_offset = (allocation.width - width) / 2;
+	y_offset = (allocation.height - height) / 2;
 
-	pix_area.x = x_offset + widget->allocation.x;
-	pix_area.y = y_offset + widget->allocation.y;
+	pix_area.x = x_offset + allocation.x;
+	pix_area.y = y_offset + allocation.y;
 	pix_area.width = width;
 	pix_area.height = height;
 
@@ -668,10 +667,10 @@ ephy_spinner_expose (GtkWidget *widget,
 		return FALSE;
 	}
 
-	gc = gdk_gc_new (widget->window);
-	gdk_draw_pixbuf (widget->window, gc, pixbuf,
-			 dest.x - x_offset - widget->allocation.x,
-			 dest.y - y_offset - widget->allocation.y,
+	gc = gdk_gc_new (gtk_widget_get_window (widget));
+	gdk_draw_pixbuf (gtk_widget_get_window (widget), gc, pixbuf,
+			 dest.x - x_offset - allocation.x,
+			 dest.y - y_offset - allocation.y,
 			 dest.x, dest.y,
 			 dest.width, dest.height,
 			 GDK_RGB_DITHER_MAX, 0, 0);
